<?php
$host = '0.0.0.0';
$port = 4370;
$logFile = __DIR__ . "/zkteco_data.log";

// Create a socket
$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
if (!$socket) {
    die("Socket creation failed: " . socket_strerror(socket_last_error()) . "\n");
}

// Set socket option to allow address reuse
socket_set_option($socket, SOL_SOCKET, SO_REUSEADDR, 1);

// Bind the socket
if (!socket_bind($socket, $host, $port)) {
    die("Socket bind failed: " . socket_strerror(socket_last_error($socket)) . "\n");
}

// Listen for incoming connections
if (!socket_listen($socket)) {
    die("Socket listen failed: " . socket_strerror(socket_last_error($socket)) . "\n");
}

echo "Listening on $host:$port...\n";

// Infinite loop to accept client connections
while (true) {
    $client = socket_accept($socket);
    if (!$client) {
        echo "Client connection failed: " . socket_strerror(socket_last_error($socket)) . "\n";
        continue;
    }

    // Read the incoming data from the client
    $data = '';
    while ($buf = @socket_read($client, 1024)) {
        $data .= $buf;
        if (strlen($buf) < 1024) break;
    }

    if ($data) {
        // Log the data to the log file, if not already logged
        $existingData = file_exists($logFile) ? file_get_contents($logFile) : '';
        if (!str_contains($existingData, $data)) { // Avoid duplicates
            file_put_contents($logFile, "[" . date("Y-m-d H:i:s") . "] Full Data:\n" . $data . PHP_EOL, FILE_APPEND);
        }

        // Extract SN (Serial Number) from the URL part of the data
        preg_match('/SN=([A-Za-z0-9]+)/', $data, $matches);
        $sn = isset($matches[1]) ? $matches[1] : null;

        // Process the second part (attendance data) only if SN is found
        if ($sn) {
            // Extract the second part (attendance data)
            $dataParts = explode("\n", $data);
            $attendanceData = array_slice($dataParts, 6); // Skipping the first 6 lines (header and headers info)

            // Process each line of attendance data
            foreach ($attendanceData as $line) {
                $line = trim($line); // Remove extra spaces
                if (empty($line)) continue; // Skip empty lines

                // Split the line by tabs or spaces
                $fields = preg_split('/\s+/', $line);

                // Ensure we have enough fields (6+ fields expected)
                if (count($fields) >= 6) {
                    // Extract data
                    $user_id = $fields[0];
                    $attendance_date = $fields[1];
                    $attendance_time = $fields[2];
                    $extra = $fields[3];
                    $punch_type = $fields[4];
                    $status_1 = $fields[5];
                    $status_2 = isset($fields[6]) ? $fields[6] : 0; // Default to 0 if not available

                    // Database connection
                    $connection = new mysqli("localhost", "hostzil_edukit", "hostzil_edukit", "hostzil_edukit");
                    if ($connection->connect_error) {
                        die("Connection failed: " . $connection->connect_error);
                    }

                    // Prepare the SQL query to insert data into the database
                    $query = "INSERT INTO attendance_log (SN, user_id, attendance_date, attendance_time, extra, punch_type, status_1, status_2) 
                              VALUES ('$sn', '$user_id', '$attendance_date', '$attendance_time', '$extra', '$punch_type', '$status_1', '$status_2')";

                    // Execute the query
                    if ($connection->query($query) === TRUE) {
                        echo "New record created successfully\n";
                    } else {
                        echo "Error: " . $query . "\n" . $connection->error;
                    }

                    // Close the database connection
                    $connection->close();
                }
            }
        }

        // Send an OK response back to the client
        $response = "HTTP/1.1 200 OK\r\nContent-Type: text/plain\r\n\r\nOK\n";
        if (@socket_write($client, $response) === false) {
            error_log("Failed to write to socket: " . socket_strerror(socket_last_error($client)));
        }
    }

    // Close the client connection
    @socket_close($client);
}

// Close the server socket
socket_close($socket);
?>
